# SportManager.py
# global variables: currentIndex, resultSet

from sqlite3 import *
from entrydialog import *
from prettytable import *

database = "schule.db"

def doFirst():
    global currentIndex
    currentIndex = 0
    showPerson(resultSet[currentIndex])
    status.setValue("Person " + str(currentIndex + 1) + " von " + str(len(resultSet)))

def doLast():
    global currentIndex
    currentIndex = len(resultSet) - 1
    showPerson(resultSet[currentIndex])
    status.setValue("Person " + str(currentIndex + 1) + " von " + str(len(resultSet)))

def doNext():
    global currentIndex
    if currentIndex == len(resultSet) - 1:
        status.setValue("Ende der Datenbank erreicht.")        
        return
    currentIndex += 1
    showPerson(resultSet[currentIndex])
    status.setValue("Person " + str(currentIndex + 1) + " von " + str(len(resultSet)))

def doPrevious():
    global currentIndex
    if currentIndex == 0:
        status.setValue("Beginn der Datenbank erreicht.")        
        return
    currentIndex -= 1
    showPerson(resultSet[currentIndex])
    status.setValue("Person " + str(currentIndex + 1) + " von " + str(len(resultSet)))

def validate(lValue, hValue, wValue):
    return True
           
def showPerson(person):
    fText.setValue(person[1])
    vText.setValue(person[2])
    personid = person[0]
    with connect(database)as con:
        cursor = con.cursor()
        sql = "SELECT * FROM sport where personid = " + str(personid)
        cursor.execute(sql)
        leistungen = cursor.fetchone()
    if leistungen == None: # Person not yet in table; should not happen
        lEntry.setValue(None)
        hEntry.setValue(None)
        wEntry.setValue(None)
    else:
        # if NULL in table, leistungen[i] = None -> empty entry
        lEntry.setValue(leistungen[1])
        hEntry.setValue(leistungen[2])
        wEntry.setValue(leistungen[3])
      
def setCurrentIndex():
    global currentIndex
    familienname = fText.getValue().strip()
    vorname = vText.getValue().strip()
    person = find(familienname, vorname)
    if person == None:    
        return False
    personid = person[0]
    indexList = [item[0] for item in resultSet]
    currentIndex = indexList.index(personid)
    return True

def search():
    global currentIndex
    familienname = fText.getValue().strip()
    vorname = vText.getValue().strip()
    person = find(familienname, vorname)
    if person == None:    
        status.setValue("Suche misslungen. Person nicht in Datenbank gefunden.")
        return
    personid = person[0]
    indexList = [item[0] for item in resultSet]
    currentIndex = indexList.index(personid)
    showPerson(resultSet[currentIndex])
    status.setValue("Suche erfolgreich. Person in Datenbank gefunden.")

# ---------------- Datenbank-Operationen ----------------
def getAllRecords():
    global resultSet
    with connect(database)as con:
        cursor = con.cursor()
        sql = "SELECT * FROM person ORDER BY familienname, vorname"
        cursor.execute(sql)
        resultSet = cursor.fetchall()

def find(familienname, vorname):
    with connect(database)as con:
        cursor = con.cursor()
        sql = "SELECT * FROM person WHERE familienname LIKE '%s' AND vorname LIKE '%s'" %(familienname, vorname)
        cursor.execute(sql)
        person = cursor.fetchone()
    return person # None, if not found    

def insert():
    lValue = lEntry.getValue()
    if lValue == None:
        lValue = 'NULL'
    hValue = hEntry.getValue()
    if hValue == None:
        hValue = 'NULL'
    wValue = wEntry.getValue()
    if wValue == None:
        wValue = 'NULL'
    if not validate(lValue, hValue, wValue):
        return
    personid = resultSet[currentIndex][0]
    with connect(database)as con:
        cursor = con.cursor()
        sql = "DELETE FROM sport WHERE personid = " + str(personid)
        cursor.execute(sql)
        sql = "INSERT INTO sport VALUES (%s, %s, %s, %s)" %(personid, lValue, hValue, wValue)
        cursor.execute(sql)
    status.setValue("Leistungen erfolgreich in Datenbank übernommen.") 

def report(discipline):
    with connect(database)as con:
        cursor = con.cursor()
        if discipline == "all":
            sql = """SELECT familienname, vorname, lauf, hochsprung, weitsprung 
                  FROM person JOIN sport 
                  ON person.personid == sport.personid 
                  ORDER BY familienname, vorname"""
        elif discipline == "lauf":
            sql = """SELECT familienname, vorname, lauf 
                  FROM person JOIN sport 
                  ON person.personid == sport.personid 
                  ORDER BY lauf ASC"""
        elif discipline == "hoch":
            sql = """SELECT familienname, vorname, hochsprung
                  FROM person JOIN sport 
                  ON person.personid == sport.personid 
                  ORDER BY hochsprung DESC"""
        elif discipline == "weit":
            sql = """SELECT familienname, vorname, weitsprung
                  FROM person JOIN sport 
                  ON person.personid == sport.personid 
                  ORDER BY weitsprung DESC"""
        cursor.execute(sql)
        printTable(cursor)
        status.setValue("Report ausgeschrieben.")
    
# ================== Hauptprogramm ===================
# --------------- Dialog aufbauen -------------
fText = StringEntry("Familienname: ") 
vText = StringEntry("Vorname: ") 
pane1 = EntryPane("Sportler/Sportlerin", fText, vText)
firstBtn = ButtonEntry("First") 
lastBtn = ButtonEntry("Last") 
nextBtn = ButtonEntry("Next") 
prevBtn = ButtonEntry("Prev") 
searchBtn = ButtonEntry("Search") 
pane2 = EntryPane(firstBtn, prevBtn, nextBtn, lastBtn, searchBtn)
lEntry = FloatEntry("Laufzeit: ") 
hEntry = FloatEntry("Hochsprung: ")
wEntry = FloatEntry("Weitsprung: ") 
pane3 = EntryPane("Leistungen", lEntry, hEntry, wEntry)
deleteBtn = ButtonEntry("Delete") 
insertBtn = ButtonEntry("Insert") 
saveBtn = ButtonEntry("Save") 
pane4 = EntryPane(saveBtn)
allBtn = ButtonEntry("Alle") 
laufBtn = ButtonEntry("Lauf") 
hochBtn = ButtonEntry("Hoch") 
weitBtn = ButtonEntry("Weit") 
pane5 = EntryPane("Reports/Ranglisten", allBtn, laufBtn, hochBtn, weitBtn)
status = StringEntry("")
status.setEditable(False)
pane6 = EntryPane("Status", status)
dlg = EntryDialog(pane1, pane2, pane3, pane4, pane5, pane6)
dlg.setTitle("Sport-Manager")

# --------------- ResultSet für ganze Datenbank holen -------------
resultSet = []
getAllRecords()
if resultSet == []:  # database empty
    status.setValue("Datenbank leer. Zuerst Persondaten eingeben.")
else:
    doFirst()
    # --------------- Ereignisschleife -------------
    while not dlg.isDisposed():
        if firstBtn.isTouched():
             doFirst()
        if lastBtn.isTouched():
             doLast()
        if nextBtn.isTouched():
             doNext()  
        if prevBtn.isTouched():
             doPrevious()  
        elif searchBtn.isTouched():
            search()
        elif saveBtn.isTouched():
            insert()
        elif allBtn.isTouched():
            report("all")
        elif laufBtn.isTouched():
            report("lauf")
        elif hochBtn.isTouched():
            report("hoch")
        elif weitBtn.isTouched():
            report("weit")
